#include "std.h"

// Jednoduchy parser, ktory testuje hlavicku dat

enum format_t 
    {unknown, flv, avi, mpg, ts, mkv, mp4}
    g_format = unknown;

bool testFLV (size_t sizeOfData, const char* data)
{
    setDataStream (sizeOfData, data);

    // FLV header:
    CMP(24, 0x464C56);
    CMP(8, 1);
    CMP(5, 0);
    SKIP(1);
    CMP(1,0);
    SKIP(1);
    CMP(32, 9);

    // FLV body:
    CMP(32,0);
    CMP(2,0);
    SKIP(1);
    int type = GET(5);
    if (! IS_VALID ())
        return true;

    switch (type)
    {
    case 8:
    case 9:
    case 18:
        break;
    default:
        return false;
    }
    SKIP(32);
    CMP(24,0);

    g_format = flv;
    return true;
}



bool testMKV (size_t sizeOfData, const char* data)
{
    setDataStream (sizeOfData, data);

    CMP(16, 0x1A45);
    CMP(16, 0xDFA3);
    CMP(1, 1);
    int count = GET(5) - 1;
    CMP(2, 3);
    for (int i=0; i<count; i++)
    {
        CMP (8, 0x42);
        int value = GET(8);
        switch (value)
        {
        case 0x86:
        case 0xF7:
        case 0xF2:
        case 0xF3:
        case 0x87:
        case 0x85:
            CMP(8,0x81);
            SKIP(8);
            break;
        case 0x82:
            CMP(8,0x88);
            CMP(8,'m');
            CMP(8,'a');
            CMP(8,'t');
            CMP(8,'r');
            CMP(8,'o');
            CMP(8,'s');
            CMP(8,'k');
            CMP(8,'a');
            break;
        default:
            return false;
        }
    }
    g_format = mkv;
    return true;

}

bool testAVI (size_t sizeOfData, const char* data)
{
    setDataStream (sizeOfData, data);

    CMP(8, 'R');
    CMP(8, 'I');
    CMP(8, 'F');
    CMP(8, 'F');
    //size_t sizeRiff = 0;
    //for (int i = 0; i < 4; i++)
    //    sizeRiff += GET(8) << (i*8);
    SKIP(32);
    CMP(8, 'A');
    CMP(8, 'V');
    CMP(8, 'I');
    int lastC = GET(8);
    if (lastC != ' ' && lastC != 'X')
        return false;

    CMP(8, 'L');
    CMP(8, 'I');
    CMP(8, 'S');
    CMP(8, 'T');
    //size_t sizeHdr1 = 0;
    //for (int i = 0; i < 4; i++)
    //    sizeHdr1 += GET(8) << (i*8);
    SKIP(32);
    CMP(8, 'h');
    CMP(8, 'd');
    CMP(8, 'r');
    SKIP(8);

    CMP(8, 'a');
    CMP(8, 'v');
    CMP(8, 'i');
    CMP(8, 'h');

    g_format = avi;    
    return true;
}

bool testMPEG (size_t sizeOfData, const char* data)
{
    setDataStream (sizeOfData, data);
    
    // PPSP:
    CMP (32, 0x000001BA);
    //CMP (16, 0x2100);
    //CMP (16, 0x0100);
    SKIP(64);

    // SH:
    CMP (32, 0x000001BB);
    int shSize = GET(16);
    if (shSize < 8)
        return false;
    if (shSize > 0x1f)
        return false;


    g_format = mpg;
    return true;
}

bool testTS (size_t sizeOfData, const char* data)
{
    int i = 0;
    for (i = 0; i < 1; i++)
    {
        setDataStream (sizeOfData-i*188, data+i*188);

        int bPUStart,PID,bAFExist,bPExist;

        // Partial Transport Steam:
        CMP (8, 0x47);
        if (! IS_VALID ())
            return true;
        if (IS_VALID () && ! GET(1)) // check error
        {
            bPUStart = GET(1);
            SKIP(1);
            PID = GET(13);
            SKIP(2);
            bAFExist = GET(1);
            bPExist = GET(1);
            SKIP(4);
            if (!IS_VALID ())
                return true;
            //Adaptation field:
            if (bAFExist)
            {
                int AFSize = GET (8);
                SKIP (AFSize*8);
            }

            if (PID==0) //PAT/PMT
            {
                int table_id = GET(8);
                if (! IS_VALID ())
                    return true;
                switch (table_id)
                {
                case 0: // PAT:
                case 2:
                case 8:
                case 34:
                    break;
                default:
                    return false;
                }
            }
            else
                return false;
        }

        sizeOfData -= 188; 
        data += 188;
        ::setDataStream (sizeOfData, data);
    }
    setDataStream(sizeOfData-i*188,data+i*188);
    CMP (8, 0x47);

    g_format = ts;
    return true;
}

//bool testTS (size_t sizeOfData, const char* data)
//{
//    int i = 0;
//    for (i = 0; i < 1; i++)
//    {
//        setDataStream (sizeOfData-i*188, data+i*188);
//
//        int bPUStart,PID,bAFExist,bPExist;
//
//        // Partial Transport Steam:
//        CMP (8, 0x47);
//        if (! IS_VALID ())
//            return true;
//        if (IS_VALID () && ! GET(1)) // check error
//        {
//            bPUStart = GET(1);
//            SKIP(1);
//            PID = GET(13);
//            SKIP(2);
//            bAFExist = GET(1);
//            bPExist = GET(1);
//            SKIP(4);
//            if (!IS_VALID ())
//                return true;
//            //Adaptation field:
//            if (bAFExist)
//            {
//                int AFSize = GET (8);
//                SKIP (AFSize*8);
//            }
//
//            if (PID==0 && bPUStart) //PAT/PMT
//            {
//                int table_id = GET(8);
//                if (! IS_VALID ())
//                    return true;
//                if (table_id==0) // PAT:
//                {
//                    SKIP(1);
//                    CMP(1,0);
//                    SKIP(2);
//                    CMP(2,0);
//                    //int len = GET(10);
//                    //if (len > 1021)
//                    //    return false;
//                }
//                else if(table_id==2)
//                {
//                    SKIP(1);
//                    CMP (3,3);
//                    CMP (2,0);
//                    if (! IS_VALID ())
//                        return true;
//                    int len = GET(10);
//                    //if (len > 1021)
//                    //    return;
//                    len = GET(16);
//                    //if (len > 1016)
//                    //    return;
//                    SKIP (2);
//                    SKIP (5);
//                    SKIP (1);
//                    CMP (16,0);
//                    //SKIP(3+13+4);
//                    //CMP (2, 0);
//                }
//            }
//            else
//                return false;
//        }
//    
//        sizeOfData -= 188; 
//        data += 188;
//        ::setDataStream (sizeOfData, data);
//    }
//    setDataStream(sizeOfData-i*188,data+i*188);
//    CMP (8, 0x47);
//
//    g_format = ts;
//    return true;
//}

bool testMP42 ()
{
    CMP (8, 'm');
    CMP (8, 'p');
    CMP (8, '4');
    int c = GET (8);
    if (! IS_VALID ())
        return true;
    if (c == '2')
        return true;
    if (c == '1')
        return true;
    return false;
}
bool testMP4 (int size, const char* data)
{
    setDataStream (size, data);
    
    CMP (24, 0);
    int count = GET(6);
    CMP (2, 0);
    CMP (8, 'f');
    CMP (8, 't');
    CMP (8, 'y');
    CMP (8, 'p');
    if (testMP42 ())
    {
        g_format = mp4;
        return true;
    }
    CMP (24, 0);
    SKIP(8);
    for (int i = 4; i < count; i++)
        if (testMP42 ())
        {
            g_format = mp4;
            return true;
        }

    return false;
}

bool test (int sizeOfData, const char* data)
{
    if (testTS (sizeOfData, data))
        return true;

    if (testMPEG (sizeOfData, data))
        return true;

    if (testAVI (sizeOfData, data))
        return true;

    if (testFLV (sizeOfData, data))
        return true;

    if (testMKV (sizeOfData, data))
        return true;

    if (testMP4 (sizeOfData, data))
        return true;

    return false;
}

int main (int argc, const char* argv[])
{
    assert (argc >= 2);
    const char* extargv [] = {argv[0], argv[1], "0", "1","1000"};
    int extargc = 5;
    bool testResult = false;

    if (parseAndValidate  (extargc, extargv))
    {
        int sizeOfData;
        const char* data;
        getData (sizeOfData, data);
        for (int i = 0; i < g_maxCountOfTests; i++)
        {
            if (testResult = test (::g_sizeofTestedData, data))
                break;
        }
    }
    const char* format = NULL;
    switch (g_format)
    {
    case flv:
        format = "flv"; break;
    case mkv:
        format = "mkv"; break;
    case mpg:
        format = "mpg"; break;
    case avi:
        format = "avi"; break;
    case ts:
        format = "ts"; break;
    case mp4:
        format = "mp4"; break;
    case unknown:
        format = "heh"; break;
    default:
        assert (0);
    }
    printf (format);
    
    return testResult ? EXIT_SUCCESS : EXIT_FAILURE;
}