#include "std.h"

#define MAX_SEGMENT_SIZE 250000

int getOffset() 
{
    size_t value = GET (8);
    if (value & 0x80)
        return value & 0x7f;
    else if (value & 0x40)
        return ((value & 0x3f)<<8) | GET(8);
    else if (value & 0x20)
        return ((value & 0x1f)<<16) | GET(16);
    else if (value & 0x10)
        return ((value & 0x0f)<<24) | GET(24);
    return 0;
}

int getCode()
{
    size_t value = GET (8);
    if (value & 0x80)
        return value;
    else if (value & 0x40)
        return ((value)<<8) | GET(8);
    else if (value & 0x20)
        return ((value)<<16) | GET(16);
    else if (value & 0x10)
        return ((value)<<24) | GET(24);
    return 0;
}

int g_lvl = 1;
#define MASTER(CODE, LVL) \
{\
    if (LVL > g_lvl+1)\
        return false;\
    g_lvl = LVL;\
    if (code == CODE)\
    {\
        int offset = getOffset();\
        if (! IS_VALID ())\
            return true;\
        if (!offset)\
            return false;\
        continue;\
    }\
}
#define VALUE(CODE, LVL)\
{\
    if (LVL > g_lvl+1)\
        return false;\
    g_lvl = LVL;\
    if (code == CODE)\
    {\
        int offset = getOffset();\
        if (! IS_VALID ())\
            return true;\
        if (!offset)\
            return false;\
        if (offset > MAX_SEGMENT_SIZE)\
            return false;\
        ::g_extraInfo = max (g_extraInfo, offset);\
        SKIP(offset*8);\
        continue;\
    }\
}
#define SVALUE(CODE, LVL)\
{\
    if (LVL > g_lvl+1)\
        return false;\
    g_lvl = LVL;\
    if (code == CODE)\
    {\
        int offset = getOffset();\
        if (! IS_VALID ()) \
            return true; \
        if (!offset)\
            return false;\
        if (offset > 8)\
            return false;\
        SKIP(offset*8);\
        continue;\
    }\
}

bool test (int size, const char* data)
{
    setDataStream (size, data);
    while (::g_streamCountBytes > 0)
    {
        int code = getCode ();
        MASTER(0x1f43b675,1);//cluster
            SVALUE(0xE7,2);//timecode
            MASTER(0x5854,2);//silenttrack
                SVALUE(0x58d7,3);//silenttracknumber
            SVALUE(0xa7,2);//position
            SVALUE(0xab,2);//prevsize
            VALUE (0xa3,2);//simpleblock
            MASTER(0xa0,2);//blockgroup
                VALUE (0xa1,3);//block
                //VALUE (0xa2);//blockvirtual
                MASTER(0x75a1,3);//blockadditions
                    MASTER(0xa6,4);//blockMore
                        SVALUE(0xee,5);//blockaddid
                        SVALUE(0xa5,5);//blockadditional
                SVALUE(0x9b,3);//blockduration
                SVALUE(0xfa,3);//referencepriority
                SVALUE(0xfb,3);//ReferenceBlock
                //SVALUE(0xfd);//ReferebceVirtual
                //VALUE(0xa4);//CodecState
                //VALUE(0x8e);//Slices

        return false;
    }

    return true;
}


int main (int argc, const char* argv[])
{
    assert (argc >= 5);
    bool testResult = false;

    stdOutput_e out = errOpen;
    if (parseAndValidate  (argc, argv))
    {
        out = fail;
        int sizeOfData;
        const char* data;
        getData (sizeOfData, data);
        for (int i = 0; i < g_maxCountOfTests; i++)
        {
            if (testResult = test (g_sizeofTestedData, data+i))
            {
                out = success;
                break;
            }
        }
    }

    stdOutput (out);
    
    return testResult ? EXIT_SUCCESS : EXIT_FAILURE;
}
